/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * $Id: Main.java 4798 2009-08-07 12:30:47Z alitokmen $
 */

package org.ow2.jasmine.monitoring.installer.deploymentplan;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * @author Alina Radu
 */
public class Main {

    /**
     * 
     */
    private Options options;

    /**
     * 
     */
    private CommandLine commandline;

    /**
     * 
     */
    private static final String MBEANCMD_OPT = "mbeancmd";

    /**
     * 
     */
    private static final String ARCHIVE_OPT = "archive";

    /**
     * 
     */
    private static final String DROOLS_OPT = "drools";

    /**
     * 
     */
    private static final String EOS_OPT = "eos";

    /**
     * 
     */
    private static final String NOTIF_OPT = "notif";

    /**
     * 
     */
    private static final String NOTIF_EXPORT_JMX_OPT = "notif_export_jmx";

    /**
     * 
     */
    private static final String NOTIF_EXPORT_SNMP_OPT = "notif_export_snmp";

    /**
     * 
     */
    private static final String REPO_OPT = "repo";

    /**
     * 
     */
    private static final String OUTPUT_OPT = "output";

    /**
    *
    */
    private static final String CONF_OPT = "conf";

    /**
    *
    */
    private static final String MBEANCMD_PORT_OPT = "mbeancmd_port";

    /**
    * 
    */
    private static final int MAX_PORT_VALUE = 65536;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    /**
     * 
     */
    public Main(final String[] args) {
        initOptions();

        final BasicParser parser = new BasicParser();

        try {
            commandline = parser.parse(options, args);
        } catch (final MissingOptionException e) {
            System.err.println(e.getMessage());
            System.exit(0);
        } catch (final ParseException e) {
            System.err.println(e.getMessage());
            System.exit(0);
        }
    }

    // ------------------------------------------------------------------------
    // Public methods
    // ------------------------------------------------------------------------

    /**
     * 
     */
    public final void generateDeploymentPlan() {

        boolean hasMbeancmd = false;
        boolean hasDB = false;
        boolean hasDrools = false;
        boolean hasEos = false;
        boolean hasNotif = false;
        boolean hasNotifExportJMX = false;
        boolean hasNotifExportSNMP = false;
        String repo = "";
        String output = "";

        if (commandline.hasOption(MBEANCMD_OPT)) {
            hasMbeancmd = Boolean.parseBoolean(commandline.getOptionValue(MBEANCMD_OPT));
        }

        if (commandline.hasOption(ARCHIVE_OPT)) {
            hasDB = Boolean.parseBoolean(commandline.getOptionValue(ARCHIVE_OPT));
        }

        if (commandline.hasOption(DROOLS_OPT)) {
            hasDrools = Boolean.parseBoolean(commandline.getOptionValue(DROOLS_OPT));
        }

        if (commandline.hasOption(EOS_OPT)) {
            hasEos = Boolean.parseBoolean(commandline.getOptionValue(EOS_OPT));
        }
        
        if (commandline.hasOption(NOTIF_OPT)) {
            hasNotif = Boolean.parseBoolean(commandline.getOptionValue(NOTIF_OPT));
        }
        
        if (commandline.hasOption(NOTIF_EXPORT_JMX_OPT)) {
            hasNotifExportJMX = Boolean.parseBoolean(commandline.getOptionValue(NOTIF_EXPORT_JMX_OPT));
        }
        
        if (commandline.hasOption(NOTIF_EXPORT_SNMP_OPT)) {
            hasNotifExportSNMP = Boolean.parseBoolean(commandline.getOptionValue(NOTIF_EXPORT_SNMP_OPT));
        }

        if (commandline.hasOption(REPO_OPT)) {
            repo = commandline.getOptionValue(REPO_OPT);
        }

        if (commandline.hasOption(OUTPUT_OPT)) {
            output = commandline.getOptionValue(OUTPUT_OPT);
        }

        new DeploymentPlanGenerator(hasMbeancmd, hasDB, hasDrools, hasEos, hasNotif, hasNotifExportJMX, hasNotifExportSNMP, repo, output);

    }

    /**
     * 
     */
    public final void generatedConfiguration() {

        int mbeancmdPort = 0;
        String confOutput = null;

        boolean hasMbeancmd = false;
        boolean hasDB = false;
        boolean hasDrools = false;
        boolean hasEos = false;

        if (commandline.hasOption(MBEANCMD_OPT)) {
            hasMbeancmd = Boolean.parseBoolean(commandline.getOptionValue(MBEANCMD_OPT));
        }

        if (commandline.hasOption(ARCHIVE_OPT)) {
            hasDB = Boolean.parseBoolean(commandline.getOptionValue(ARCHIVE_OPT));
        }

        if (commandline.hasOption(DROOLS_OPT)) {
            hasDrools = Boolean.parseBoolean(commandline.getOptionValue(DROOLS_OPT));
        }

        if (commandline.hasOption(EOS_OPT)) {
            hasEos = Boolean.parseBoolean(commandline.getOptionValue(EOS_OPT));
        }

        if (commandline.hasOption(MBEANCMD_PORT_OPT)) {
            final String value = commandline.getOptionValue(MBEANCMD_PORT_OPT);
            try {
                final int port = Integer.parseInt(value);
                if ((port > 0) && (port < MAX_PORT_VALUE)) {
                    mbeancmdPort = port;
                }
            } catch (final NumberFormatException ignored) {
            }
        }

        if (commandline.hasOption(CONF_OPT)) {
            confOutput = commandline.getOptionValue(CONF_OPT);
        }

        new ConfigurationGenerator(mbeancmdPort, confOutput, hasMbeancmd, hasDB, hasDrools, hasEos);
    }

    // ------------------------------------------------------------------------
    // Main method
    // ------------------------------------------------------------------------

    /**
     * @param args
     */
    public static void main(final String[] args) {

        final Main main = new Main(args);

        main.generatedConfiguration();
        main.generateDeploymentPlan();
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Initialize command line options
     */
    private void initOptions() {

        options = new Options();

        OptionBuilder.withArgName(MBEANCMD_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("MBeanCmd must be installed");
        // mbeancmd
        final Option mbeancmd = OptionBuilder.create(MBEANCMD_OPT);
        options.addOption(mbeancmd);

        OptionBuilder.withArgName(ARCHIVE_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("Archive feature must be installed");
        // archive in DB
        final Option archive = OptionBuilder.create(ARCHIVE_OPT);
        options.addOption(archive);

        OptionBuilder.withArgName(DROOLS_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("Drools engine must be installed");
        // Drools
        final Option drools = OptionBuilder.create(DROOLS_OPT);
        options.addOption(drools);

        OptionBuilder.withArgName(EOS_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("JASMINe EoS must be installed");
        // eos
        final Option eos = OptionBuilder.create(EOS_OPT);
        options.addOption(eos);

        OptionBuilder.withArgName(NOTIF_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("The notification system must be installed");
        // notif
        final Option notif = OptionBuilder.create(NOTIF_OPT);
        options.addOption(notif);

        OptionBuilder.withArgName(NOTIF_EXPORT_JMX_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("The notifications will be exported as JMX notifications");
        // notif export JMX
        final Option notifExportJMX = OptionBuilder.create(NOTIF_EXPORT_JMX_OPT);
        options.addOption(notifExportJMX);

        OptionBuilder.withArgName(NOTIF_EXPORT_SNMP_OPT);
        OptionBuilder.isRequired();
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("The notifications will be exported as SNMP traps");
        // notif export SNMP
        final Option notifExportSNMP = OptionBuilder.create(NOTIF_EXPORT_SNMP_OPT);
        options.addOption(notifExportSNMP);

        OptionBuilder.withArgName(REPO_OPT);
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("Repository containing modules to install");
        // repository
        final Option repo = OptionBuilder.create(REPO_OPT);
        options.addOption(repo);

        OptionBuilder.withArgName(OUTPUT_OPT);
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("Location of the file generated");
        // output
        final Option output = OptionBuilder.create(OUTPUT_OPT);
        options.addOption(output);

        // mbeancmd port
        final Option mbeancmdPort = new Option(MBEANCMD_PORT_OPT, true, "MBeanCmd port");
        options.addOption(mbeancmdPort);

        OptionBuilder.withArgName(CONF_OPT);
        OptionBuilder.hasArg();
        OptionBuilder.withDescription("Location of the configuration file generated");
        // conf
        final Option conf = OptionBuilder.create(CONF_OPT);
        conf.setRequired(true);
        options.addOption(conf);

    }

}
